﻿using System;
using System.Data;
using System.Threading;
using System.Windows.Threading;

namespace SampleApp
{
  public class CustomerDataSet
  {
    #region Private Declarations
    private const int NumColumns = 180;
    private const string ColHeader = "Column ";
    private const string ParentHeader = "Parent";
    private const string PrimaryHeader = "Id";
    private const string RelationshipName = "hierarchy";
    private DataTable _customer;
    private DataTable _orders;
    private readonly DataSet _dataSet;
    private bool _isDisposed = false;
    #endregion Private Declarations

    #region Public Properties
    public DataSet DataSet
    {
      get
      {
        return _dataSet;
      }
    }

    public DataTable Customer
    {
      get
      {
        return _customer;
      }
    }

    public DataTable Orders
    {
      get
      {
        return _orders;
      }
    }
    #endregion


    // CHANGES start*********************************
    private Random _random = new Random();
    private DispatcherTimer _timer = new DispatcherTimer();
    int moreRows = 200;
    bool createChildTable_sw = false;
    // CHANGES END **********************************


    #region Constructors
    Random random = new Random(100);
      
    public CustomerDataSet(bool createChildTable_)
    {
      CreateTestTables(createChildTable_);

      _dataSet = new DataSet("CustomerDataSet");
      _dataSet.Tables.Add(_customer);

      if (createChildTable_)
      {
        _dataSet.Tables.Add(_orders);

        var tableRelation = new DataRelation(RelationshipName, _dataSet.Tables["Customers"].Columns[PrimaryHeader], _dataSet.Tables["Orders"].Columns[ParentHeader]);
        _dataSet.Relations.Add(tableRelation);
      }
      _dataSet.AcceptChanges();
      Dispatcher dispatcher = Dispatcher.CurrentDispatcher;
      

      // CHANGES start *******************************
      //int moreRows = 200;
      // CHANGES end *********************************


      // CHANGES start *******************************
      createChildTable_sw = createChildTable_;
      if (!_timer.IsEnabled)
      {
          // Set up time interval
          _timer.Interval = new TimeSpan(0, 0, 0, 0, 500);

          // When time interval has elapsed, re-call
          _timer.Tick += new EventHandler(_timer_Tick);
          _timer.Start();
      }
      // CHANGES end *********************************


      // CHANGES start *******************************
      //ThreadPool.QueueUserWorkItem((object none) =>
      //{
      //    while (!_isDisposed)
      //    {
      //        Thread.Sleep(500);
      //        try
      //        {
      //            dispatcher.BeginInvoke((Action)(() =>
      //            {
      //                _customer.BeginLoadData();
      //                ApplyRandomUpdate(_customer);
      //                if (moreRows > 0) // Add few rows (simulate sending new orders realtime)
      //                {
      //                    CreateRow(_customer.Rows.Count, _customer, -1);
      //                    moreRows--;
      //                }
      //                _customer.EndLoadData();
      //                _customer.AcceptChanges();

      //                if (createChildTable_)
      //                {
      //                    _orders.BeginLoadData();
      //                    ApplyRandomUpdate(_orders);
      //                    _orders.EndLoadData();
      //                }
      //            }), null);
      //        }
      //        catch (Exception ex)
      //        {
      //            // Should not be hit.
      //        }
      //    }
      //});
      // CHANGES end **********************************

    }

    // CHANGES start **********************************
    void _timer_Tick(object sender, EventArgs e)
    {
        _customer.BeginLoadData();
        ApplyRandomUpdate(_customer);
        if (moreRows > 0) // Add few rows (simulate sending new orders realtime)
        {
            CreateRow(_customer.Rows.Count, _customer, -1);
            moreRows--;
        }
        _customer.EndLoadData();
        _customer.AcceptChanges();

        if (createChildTable_sw)
        {
            _orders.BeginLoadData();
            ApplyRandomUpdate(_orders);
            _orders.EndLoadData();
        }
    }
    // CHANGES end*************************************


    public void Dispose()
    {
      _isDisposed = true;
    }

    private void ApplyRandomUpdate(DataTable table_)
    {
        foreach (DataRow dr in table_.Rows)
        {
            {
                int nextRandom = random.Next();
                int anotherRandom = random.Next();
                dr.BeginEdit();
                for (int i = 1; i < 50; i++)
                {
                    if ((i + anotherRandom) % 2 == 0) // only update half of cells in the row. so only 25 cells are update with each update.
                    {
                        dr[i] = nextRandom + i;
                    }
                }
                dr.EndEdit();
            }
        }
    }
    #endregion

    #region Private Methods

    private static void CreateRow(int id_, DataTable dataTable_, int parentId_)
    {
      var row = dataTable_.NewRow();

      row[PrimaryHeader] = id_;
      for (int i = 1; i < NumColumns; i++)
      {
        row[i] = "Row: " + id_ + " Col:" + i;
      }

      if (parentId_ >= 0)
        row[ParentHeader] = parentId_;

      dataTable_.Rows.Add(row);
    }

    private static void CreateColumns(DataTable dataTable_, bool isChildTable_)
    {
      var column = new DataColumn(PrimaryHeader, typeof(Int32));
      dataTable_.Columns.Add(column);
      for (int i = 1; i < NumColumns; i++)
      {
        column = new DataColumn(ColHeader + i, typeof(String));
        dataTable_.Columns.Add(column);
      }

      if (isChildTable_)
      {
        column = new DataColumn(ParentHeader, typeof(Int32));
        dataTable_.Columns.Add(column);
      }
    }

    private void CreateTestTables(bool createChildTable_)
    {
      _customer = new DataTable("Customers");
      CreateColumns(_customer, false);

      int orderCount = 0;
      int totalRows = 400;

      if (createChildTable_)
      {
        _orders = new DataTable("Orders");
        CreateColumns(_orders, true);

        totalRows = totalRows * 5; //2000
        for (int i = 0; i < totalRows; i++)
        {
          CreateRow(i, _customer, -1);
          if (i < 5) // Only 5 records have child row. Remove this condition to have child row for each records.
          {
            for (var x = 0; x < 5; x++)
            {
              CreateRow(orderCount, _orders, i);
              orderCount++;
            }
          }
        }
      }
      else
      {
        for (int i = 0; i < totalRows * 5; i++)
        {
          CreateRow(i, _customer, -1);
        }
      }

      _customer.AcceptChanges();
      if (createChildTable_)
      {
        _orders.AcceptChanges();
      }
    }

    #endregion

  }
}
